﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Data;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Imaging;
using System.Windows.Navigation;
using System.Windows.Shapes;
using System.Collections.ObjectModel;
using Infragistics.Windows.DataPresenter;

namespace Sample1
{
  /// <summary>
  /// Interaction logic for Window1.xaml
  /// </summary>
  public partial class Window1 : Window
  {
    private Dictionary<string, bool> FieldShown { get; set; }

    public ObservableCollection<SettingItem> FilteredSettingItems { get; set; }
    public ObservableCollection<Field> ShownColumns { get; private set; }

    private Field _testItem;

    public Window1()
    {
      InitializeComponent();
      this.DataContext = this;

      InitializeFieldShownCollection();

      this.FilteredSettingItems = new ObservableCollection<SettingItem>();


      SubSetting newSubSetting = new SubSetting()
        {
          Count = "1",
          Degree = "2",
          Description = "Test",
          OffenseDate = "11/18/08",
          Statute = "1.234.78"
        };

      Setting newSetting = new Setting() 
        {
          StartTime = "4:00 PM",
          CaseNbr = "1234567"
        };

      SettingItem newSettingItem = new SettingItem()
        {
          Setting = newSetting
        };

      newSettingItem.SubSettings.Add(newSubSetting);

      this.FilteredSettingItems.Add(newSettingItem);

      this.ShownColumns = new ObservableCollection<Field>();
      PopulateWithAllValidFields(this.ShownColumns);

      this._testItem = xamSettingHierarchyGrid.FieldLayouts[0].Fields["InterpreterLanguageID"];
    }

    private void Button_Click(object sender, RoutedEventArgs e)
    {
      if (this.ShownColumns.Contains(this._testItem))
      {
        int oldIndex = this.ShownColumns.IndexOf(this._testItem);

        if (oldIndex > 0)
        {
          this.ShownColumns.Move(oldIndex, oldIndex - 1);
        }
      }

      RefreshColumnsForDynamicMode();
    }

    private void RefreshColumnsForDynamicMode()
    {
      int positionIndex = 1; //start at 1, b/c the IsSelected column is at the zero index

      foreach (Field currentField in this.ShownColumns)
      {
        MoveFieldToIndex(currentField.Name, positionIndex);
        ShowField(currentField.Name, true);
        positionIndex++;
      }
    }

    public void PopulateWithAllValidFields(ObservableCollection<Field> fieldCollection)
    {
      foreach (Field currentField in xamSettingHierarchyGrid.FieldLayouts[0].Fields)
      {
        if (currentField.Name != "SubSettings" && currentField.Name != "IsSelected" && currentField.Name != "Status")
        {
          fieldCollection.Add(currentField);
        }
      }
    }

    public bool IsFieldShown(string fieldname)
    {
      if (this.FieldShown.ContainsKey(fieldname))
      {
        return this.FieldShown[fieldname];
      }

      return false;
    }

    public void ShowField(string fieldname, bool show)
    {
      if (this.FieldShown.ContainsKey(fieldname) && this.FieldShown[fieldname] != show)
      {
        this.FieldShown[fieldname] = show;

        if (show)
        {
          xamSettingHierarchyGrid.FieldLayouts[0].Fields[fieldname].Visibility = Visibility.Visible;
        }
        else
        {
          xamSettingHierarchyGrid.FieldLayouts[0].Fields[fieldname].Visibility = Visibility.Collapsed;
        }
      }
    }

    private void InitializeFieldShownCollection()
    {
      this.FieldShown = new Dictionary<string, bool>();
      foreach (Field currentField in xamSettingHierarchyGrid.FieldLayouts[0].Fields)
      {
        //by default, we show everything, so set all to true.
        this.FieldShown.Add(currentField.Name, true);
      }
    }

    private void MoveFieldToIndex(string fieldname, int moveToIndex)
    {
      //Check if it's in the fieldshown dictionary, b/c we know it's in the grid that way.
      if (this.FieldShown.ContainsKey(fieldname))
      {
        int startIndex = xamSettingHierarchyGrid.FieldLayouts[0].Fields[fieldname].Index;

        if (startIndex != moveToIndex)
          xamSettingHierarchyGrid.FieldLayouts[0].Fields.Move(startIndex, moveToIndex);
      }
    }
  }
}
